	page 		65,132
	title		LCD_PIO.ASM
	subttl		Copyright JK microsystems 2004
	Code_seg	Segment
	assume		cs:code_seg,ds:code_seg,ss:code_seg

;************************************************************************;
;*								       	*;
;*	Copyright JK microsystems 2004	-  All Rights Reserved	  	*;
;*									*;
;*	This software is NOT shareware, freeware, or public domain.	*;
;*	It is the property of JK microsystems.				*;
;*									*;
;*	Customers of JK microsystems may modify the source code		*; 
;*	and/or distribute the binary image of this software without 	*;
;*	additional costs provided it is run only on hardware  		*;
;*	manufactured by JK microsystems.  All other use is expressly 	*;
;*	prohibited.							*;
;*									*;
;*	Update Log							*;
;*									*;
;*	Version	Date	 Comments			 Progammer	*;
;*	-------	------	 ------------------------------- ----------	*;
;*	1.0	040604	 First release, diverged from 	 jds		*;
;*			 LCD_UF.asm					*;
;*									*;
;************************************************************************;
	
	org	100h
start:					; Put the initialization code at
	jmp	init			; the end so we don't have to save it
	
;-----------------------------------------------------------------------------
;
;	This is invoked with each bios LPT interrupt
;
LPT_int:
	sti				; interrupts back on
	cmp	dx,0			; LPT number greater than 1
	je	do_lcd			; no, do it
	jmp	chain			; yes, chain to old handler

do_lcd:
	push	dx			; save a couple registers
	push	bx

	push	ax			; point ds to the bios data 
	push	ds			; area 
	mov	ax,40h			; so we can get the out register
	mov	ds,ax

	mov	ax,ds:[0B0h]		; get the lower word 
	mov	cs:[lo_word],ax		; store it locally
	mov	al,ds:[0B2h]		; get the upper byte
	mov	cs:[hi_byte],al		; store it locally
	mov	ds:[0B3h],byte ptr 1	; set busy flag	

	pop	ds			; restore registers
	pop	ax

	cmp	al,0A0h			; command escape char?
	jne	LPT1_not_esc		; no, go around
	mov	cs:[LPT1_cmd_flag],1	; yes, set the flag
	jmp	lpt_done		; and return

LPT1_not_esc:
	mov	bl,al			; save the char
	shr	al,1			; move the first nibble over	
	shr	al,1
	shr	al,1
	shr	al,1
	cmp	cs:[lpt1_cmd_flag],1	; was it a command?
	je	send_LPT1_cmd		; yes, go elsewhere to send it
	call	data_write		; no, it's data so send it here
	mov	al,bl			; restore the data
	call	data_write		; send the low nibble
	jmp	lpt_done		; and leave

send_LPT1_cmd:
	call	cmd_write		; come here to send it as a command
	mov	al,bl			; restore the data
	call	cmd_write		; and send the low nibble
	mov	cs:[lpt1_cmd_flag],0	; clear the flag
LPT_done:
	push	ax			; point ds to the bios data 
	push	ds			; area 
	mov	ax,40h			; so we can clear the busy flag
	mov	ds,ax
	mov	ds:[0B3h],byte ptr 0	; clear busy flag	
	pop	ds
	pop	ax

	mov	ah,0D0h			; return good status
	pop	bx
	pop	dx			

	iret

chain:
	push	cs:[old_LPT_seg]	; pushing these and leaving with
	push	cs:[old_LPT_off]	; a retf will chain to old lpt
	retf				; routine

;----------------------------------------------------------------------------
;
;	Delay for 5ms

delay_5_ms:
	push 	ax			; ax gets trashed
	mov 	ax,1000			; time constant for 5ms

delay_5_ms_loop:
	dec	ax			; do the loop
	jnz	delay_5_ms_loop
	pop	ax
	ret
		
;----------------------------------------------------------------------------
;
;	Set the bus to idle state

bus_idle:
	and	cs:[lo_word],word ptr 0FF8Fh 	; clear RS,R/W,E
	call	load_chip			; make it so
	call	delay_5_ms	
	ret

;----------------------------------------------------------------------------
;
;	Write the data on the bus to the LCD

write:
	or	cs:[lo_word],word ptr 40h	; make E high
	call	load_chip
	and	cs:[lo_word],word ptr 0FFBFh	; make E low
	call	load_chip
	ret 

;----------------------------------------------------------------------------
;
;	Write a command to the lcd

cmd_write:
	call	enable_data
	call	write
	call	bus_idle
	ret

;----------------------------------------------------------------------------
;
;	Write data to the lcd specified in ah

data_write:
	call	enable_data
	or	cs:[lo_word],word ptr 10h	; make RS high
	call	write
	call	bus_idle
	ret
	
;-------------------------------------------------------------------------
;
;	Enable data to LCD.  Put the contents of AL 0-3 
;	in lo_word 7-10, clock it into the NJU3718 chip
;	and enable it to LCD

enable_data:
	mov	ah,al			; move 0-3 to 8-11
	shr	ax,1			; move to 7-10
	and	ax,780h			; zero all but 7-10

	and	cs:[lo_word],word ptr 0F87Fh
	or	cs:[lo_word],ax

	call	load_chip
	ret
	
;-----------------------------------------------------------------------------
;
;	Load Chip - local routine to move hi_byte and lo_word to
;	NJU3718 chip.

load_chip:
	push	ax
	push	bx
	push	cx
	push	dx

	mov	bl,cs:[hi_byte]

	shl	bl,1			; left-justify them
	shl	bl,1
	shl	bl,1
	shl	bl,1

	mov	dx,604h			; point to I/O port
	mov	al,0Ch			; CLR/ STB/ CLK DATA
	out	dx,al			;  1    1    0   0
	mov	cx,4
hi:
	rcl	bl,1			; get current bit into carry
	rcl	al,1			; get carry into al, bit 0
	and	al,1			; clear all but bit 0
	or	al,0Eh			; set CLR/ STB/ CLK
	out	dx,al			; send it
	and	al,0FDh			; clear CLK
	out	dx,al			; send it
	loop	hi			; loop through rest

	mov	bx,cs:[lo_word]		

	mov	cx,16			; do all 16 bits
lo:
	rcl	bx,1			; get current bit into carry
	rcl	al,1			; get carry into al, bit 0
	and	al,1			; clear all but bit 0
	or	al,0Eh			; set CLR/ STB/ CLK
	out	dx,al			; send it
	and	al,0FDh			; clear CLK
	out	dx,al			; send it
	loop	lo			; loop through rest

	mov	al,8
	out	dx,al			; strobe the data to the outputs
	mov	al,0Ch			; make STB/ high
	out	dx,al

	pop	dx
	pop	cx
	pop	bx
	pop	ax

	ret	

;-----------------------------------------------------------------------------
;
;	Data and equates	
;
old_LPT_seg	dw	0		; save the old LPT routine
old_LPT_off	dw	0		; vectors in case we need them

LPT1_cmd_flag	db	0		; 1 if next byte is a command

lo_word		dw	?
hi_byte 	db	?

last_add:				; last address to save when terminating

;-----------------------------------------------------------------------------
;
;	This is our initialization code
;
init:

	mov	ax,40h			; point to bios data area
	mov	ds,ax

	mov	ax,ds:[10h]		; get equipment word
	or	ah,010000000B		; bits 14 and 15 = number of printers
	and	ah,001111111B		; clear bit 15, set bit 14, 1 printer
	mov	ds:[10h],ax		; put equipment word back

	mov	ax,0			; point to interrupt vector table
	mov 	ds,ax

	cli				; no ints while changing vectors

	mov	ax,ds:[5Ch]		; get int 17h offset
	mov	cs:[old_LPT_off],ax	; save it
	mov	ds:[5Ch],offset LPT_int ; and replace it with our routine
	mov	ax,ds:[5Eh]		; get int 17h segment
	mov	cs:[old_LPT_seg],ax	; save it
	mov	ax,cs			; get our segment
	mov	ds:[5Eh],ax		; and install it

	sti				; ints back on

	mov	dx,606h			; point to config reg
	in	al,dx			; get it
	or	al,8			; set  bit 3
	out	dx,al			; send it back to cpld
	call	bus_idle		; set bus_idle state

; ---------- LCD init code starts here

	mov	cx,3			; send 3 03 commands to lcd 0
init_loop0:
	call	delay_5_ms
	mov	ah,0
	mov	al,3
	call	cmd_write
	loop	init_loop0

	call	delay_5_ms		; give it some time to settle
	mov	ah,0			; lcd 0
	mov	al,2			; set to 4 bit bus
	call	cmd_write		; write it

	mov	cx,8			; init string is 8 bytes
	mov	di,offset init_string	; clear the lcd and home the cursor
clear_loop0:
	mov	dx,0			; lcd 0
	mov	al,cs:[di]

	jmp	tsr

	int	17h
	inc	di
	loop 	clear_loop0

; ---------- LCD init code ends here

tsr:
	mov	ax,3100h		; TSR function code
	mov 	dx,offset last_add 	; get last address used
	mov 	cl,4			; convert to paragraphs
	shr     dx,cl			; by dividing by 16
	inc	dx			; and rounding up by one

	int	21h			; tell dos to do it

init_string:
	db	160,40			; 2 lines
 	db	160,12			; display on
	db	160,6			; position increment, no shift
	db	160,1			; clear screen, home cursor

	code_seg 	ends
	end		start
