#include <stdio.h>
#include <conio.h>

#include "pio_drvr.h"

#define CPV 819.0		// counts per volt = 4095/Vin,max = 4095/5

// table of resistances in 5deg C increments from -40 to +140
// for 100k thermistor
float Rt[]=		 {	4397.0,	3088.0,	2197.0,	1581.0,	1151.0,
					846.6, 	629.0, 	471.6,	357.0,	272.5,
					209.7,	162.7,	127.1,	100.0,	79.22,
					63.17,	50.68,	40.90,	33.19,	27.09,
					22.22,	18.32,	15.18,	12.64,	10.57,
					8.873,	7.481,	6.337,	5.383,	4.594,
					3.934,	3.380,	2.915,	2.522, 	NULL };

float temperature ( float Vin ) {

float temp = -40.0, resistance;
int i=0;

		resistance= (Vin > 0) ? ((4.096-(1.2206*Vin))/(Vin*0.01212)) : 0;

//		from the schematic, Vin=4.096 - Vthermistor - Vparallel resistors
//		Vin also = V across 82.5k resistor.
//		so, the current through the loop is Vin/82.5k = Vin*0.01212
//		since we want k ohms, we omit the k from the equation.
//		now, V of the thermistor = 4.096 - (i * 18.2k) +Vin.
//		some subsitiutions give Vtherm=4.096-(1.2206 * Vin)
//		we want the resistance, so divide by i
// 		also check for Vin = 0 so we don't divide by zero

		while (Rt[i] > resistance && Rt[i] != NULL ) {	// find the resistance and temp from table
			i++;										// be sure we don't walk off the table
			temp+= 5.0;
		}

		if (i != 0 && Rt[i] != NULL ) {					// if we have something, interpolate
														// between the table values
			return temp + ( (Rt[i] - resistance) * (5.0/(Rt[i-1]-Rt[i])) );
		}
		else if( i==0)									// else error low
			return 999.99;
		else
			return 999.99;								// else error high
}


int main ( void ) {
	unsigned long relay=0;
	int i;
	unsigned long j;
	char c;
	int set=0, clear=0;
	int count=1, val=0;
	int ad;

	printf( "%c[2J%c[H", 27, 27 );		// clear screen and home cursor
	puts("PicoIO Test Program");
	InitIO();							// init i/o board
	printf("Driver version is %d.%d\n\n",(GetVersion()/10),(GetVersion() % 10) );
	puts("Enter Sxx<enter> or Cxx<enter> to set or clear relays (xx=0-19)\n");
	puts("             Inputs                      Outputs");
	puts("3322222222221111111111            1111111111");
	puts("10987654321098765432109876543210  98765432109876543210");
	puts("--------------------------------  --------------------");

	while ( 1 ) {

		if (kbhit() ){
			c=getch();
			if (c=='q' || c=='Q') {
				printf( "%c[18B",27);		// down 18 lines
				break;
			}

			switch(c) {
				case 's':  case 'S':
					set=1;
					break;

				case 'c': case 'C':
					clear=1;
					break;

				case '0': case '1': case '2': case '3': case '4':
				case '5': case '6': case '7': case '8': case '9':
					val=(val*count)+(c-0x30);
					count*=10;
					break;

				case '\r': case '\n':
					if (val > 19 || val < 0) {
						set=0; clear=0;
					}
					if (set) {
						SetIOPt(val);
						relay |= 1L << val;
					}
					if (clear) {
						ClrIOPt(val);
						relay &= ~(1L << val);
					}
					// break intentionally omitted, want to fall through and reset values
				case 0x1B:				// ESC
				default:
					clear=0;
					set=0;
					val=0;
					count=1;
					break;
			}
		}

		for (i=31; i>=0; i--) {
			printf("%c", (GetIOPt(i) > 0 ? '1' : '0') );	// get/print input status
		}
		printf("  ");
		for (j=0x80000L; j>0x00; j=j>>1)
			putch( (relay & j ? '1' : '0'));				// print relay status

		printf("\n\nCh  Counts  Volts");
		for (i=0; i<11; i++) {
			ad=GetAD(i);
			printf("\n%2d    %4d  %5.3f", i, ad, ad/CPV );	// get/print A/D valules
		}
		printf("  %6.2f degC\n", temperature(ad) );

		printf( "%c[80D",27);		// go to left of screen
		printf( "%c[14A",27);		// up 14 lines

	}

	return 0;
}
