/*
 * UTIL.C
 *
 * Embedded Web Server
 * Copyright 1998,
 * JK microsystems, Inc. - ALL RIGHTS RESERVED
 * http://www.jkmicro.com
 *
 * THIS SOFTWARE IS NOT SHAREWARE, FREEWARE, OR PUBLIC DOMAIN.
 * IT IS THE PROPERTY OF JK microsystems.
 *
 * CUSTOMERS OF JK microsystems MAY MODIFY THE SOURCE CODE
 * AND/OR DISTRIBUTE THE BINARY IMAGE OF THIS SOFTWARE WITHOUT
 * ADDITIONAL COSTS PROVIDED IT IS RUN ONLY ON HARDWARE
 * MANUFACTURED BY JK microsystems.  ALL OTHER USE IS EXPRESSLY
 * PROHIBITED.
 *
 * THIS SOURCE CODE IS NOT TO BE DISCLOSED WITHOUT PRIOR APPROVAL
 * FROM JK microsystems.
 *
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY;
 * WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY
 * OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * THE TCP/IP LIBRARIES ARE COPYRIGHT
 * UNIVERSITY OF WATERLOO.
 * THE LIBRARIES ARE PROVIDED FREE OF
 * CHARGE IN THEIR ORIGINAL DISTRIBUTION FORMAT.
 *
 */

/*
   See WEBTCP.H for revision notes
*/

#pragma option -ml
#pragma option -r-
#pragma option -2				// 286 instruction set (may be probs w/ 386 regs and TSRs)


#include "util.h"
#include "pio_drvr.h"

static char *dayname[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
static char *monthname[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
							 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

void fix_filename ( char *fn )
{
		while ( *fn != '\0' ) {
				if (*fn == '\\')
						*fn = '/';
				fn++;
		}
		return;
}

// determine mime type of file given extension
// input:       pointer to extension
// output:      buffer containing mimetype string
//              integer file type (from defined list)

void get_filetype ( char *ext, char *buffer, int *filetype)
{
		if ( !strcmp( ext, ".GIF" ) ) {
				*filetype = GIF;
				strcpy( buffer, "image/gif\0");
		}
		else if ( !strcmp( ext, ".JPG" ) ) {
				*filetype = JPG;
				strcpy( buffer, "image/jpeg\0");
		}
		else if ( !strcmp( ext, ".HTM" ) ) {
				*filetype = HTML;
				strcpy( buffer, "text/html\0");
		}
		else if ( !strcmp( ext, ".TXT" ) ) {
				*filetype = TXT;
				strcpy( buffer, "text/plain\0");
		}
		else if ( !strcmp( ext, ".PDF" ) ) {
				*filetype = PDF;
				strcpy( buffer, "application/pdf\0");
		}
		else if ( !strcmp( ext, ".ZIP" ) ) {
				*filetype = ZIP;
				strcpy( buffer, "application/zip\0");
		}
		else if ( !strcmp( ext, ".SHT" ) ) {
				*filetype = HTML;
				strcpy( buffer, "text/html\0");
		}
		else  {
				*filetype = DEF;
				strcpy( buffer, "*/*\0");
//				printf("file extension: %s ", fileext);
		}
}

// string must be of form port=(A,B,DIR_AB,U1,U1D,U3,U3D)&v=xx&v=xx&act=(READ,SET,CLEAR)
// xx: 0..9, A..F, hex value
// multiple V= strings are ANDed togetor to generate a mask for the operation

int proc_hdwr ( char *buffer )
{
unsigned int port;           	// I/O port location
unsigned char value=0;      	// operation Bit Mask
unsigned char c;				// 8bit port data
char *ptr;              		// parsing pointer

		strupr( buffer );   // make it upper case

		if( (ptr=strstr(buffer,"PORT")) == NULL )
				return (NOT_FOUND);             // request failed

		ptr+=5;         // advance past 'port='
		switch (*ptr) {
				case 'A':
						port=PORT_A;				// flashTCP port A
						break;
				case 'B':
						port=PORT_B;              	// flashTCP port B
						break;
				case 'D':
						port=PORT_AB_DIR;           // flashTCP port AB direction
						break;
				case 'R':
						port=PORT_RELAY_IO;			// flashTCP relay IO board
						break;
				case 'U':
						switch (*(ptr+1)) {
							case '1':
								if ( (*(ptr+2)) == 'D' )
									port = PORT_UF1D;		// uflash port 1 direction
								else
									port = PORT_UF1;		// uflash port 1
								break;
							case '3':
								if ( (*(ptr+2)) == 'D' )
									port = PORT_UF3D;		// uflash port 2 direction
								else
									port = PORT_UF3;		// uflash port 2
								break;
							case 'I': 						// uflash + uio drivers
								port = UF_UIO;
								break;
						}
						break;
				default:
						return (NOT_FOUND);     // request failed
		}

		while ( (ptr = strstr(ptr,"V=")) != NULL && *ptr != '\0' ) {
				ptr+=2;         // advance past 'V='

				// compile bitmask
				value |= ( (*ptr >= 'A' && *ptr <= 'F') ? *ptr-0x37 : *ptr - 0x30) << 4;
				value |= (*(ptr+1) >= 'A' && *(ptr+1) <= 'F') ? *(ptr+1)-0x37 : *(ptr+1) - 0x30;

				if ( port==PORT_UF1 ) {
					value &= PORT_UF1_MASK;
				}
				if ( port==PORT_UF3 ) {
					value &= PORT_UF3_MASK;
				}
		}
		ptr = (strstr(buffer,"ACT"));

		if (port != UF_UIO){		// don't do this stuff if request targeting UIO
			c=inportb(port);
			if (port == PORT_UF1 || port == PORT_UF3) port+=2;
		}

		if (*(ptr+4) == 'R') {
				printf(" read ");              // nothing else yet.
		}

		else if (*(ptr+4) == 'S' ) {
			if (port == UF_UIO) {
				for (c=0x01,port=0; port < 4; c=c<<1,port++) {
					if (value & c) {
						SetIOPt( (int)port );		/* turn selected driver on */

					}
				 }
			}
			else {
				c |= value;
				outportb( port, c );
			}
		}
		else if (*(ptr+4) == 'C' ) {
			if (port == UF_UIO) {
				for (c=0x01,port=0; port < 4; c=c<<1,port++) {
					if (value & c) {
						ClrIOPt( (int)port );		/* turn selected driver off */
					}
				 }
			}
			else {
				c &= ~value;
				outportb( port, c );
			}
		}
		else {
				;
		}

		return(NO_CONTENT);     // good return code, browser shouldn't
								// change screens or give error
}



void print_all (char *buf, FILE *out )     // print string displaying /n and /r
{                                          // sends to open file
	if(out==NULL) return;

		while (*buf != '\0' ) {
				if (*buf == '\r')
						fprintf( out, "\\r");
				else if (*buf == '\n' )
						fprintf( out, "\\n\n");
				else
						putc(*buf, out);
				buf++;
		}
}


void make_web_time (char * wt )         // format time/date per RFC-822
{
struct tm *tp;
time_t sec_now;

		time(&sec_now);
		tp = gmtime(&sec_now);

		sprintf( wt, "Date: %3s, %02d %03s %4d %02d:%02d:%02d GMT\r\n\0",
				dayname[tp->tm_wday], tp->tm_mday, monthname[tp->tm_mon],
				tp->tm_year + 1900, tp->tm_hour, tp->tm_min, tp->tm_sec );

//        printf("%s\n",wt);
}

char * make_local_time (char * wt )         // create string for local time/date
{
struct tm *tp;
time_t sec_now;

		time(&sec_now);
		tp = localtime(&sec_now);

		sprintf( wt, "%3s, %02d %03s %4d %02d:%02d:%02d\0",
				dayname[tp->tm_wday], tp->tm_mday, monthname[tp->tm_mon],
				tp->tm_year + 1900, tp->tm_hour, tp->tm_min, tp->tm_sec );
		return wt;
}

// use int15h to verify JK micro company ID in MAC
int flashlite( unsigned int mac)
{
		if ( !pc ) {
//			regs.w.ax= MAC_FCN;
//			int86( IO_EXT_INT, &regs, &regs );
//			if ((regs.w.bx == MAC_HI) && ((regs.w.cx & 0x00FF) == MAC_LO))
//				return (1);

//		get MAC from packet driver so it will work w/ uFlash and 386Ex
			if ((_eth_addr[1] == ((MAC_HI & 0xFF00) >> 8)) && (_eth_addr[0] == (MAC_HI & 0x00FF)) &&
				(_eth_addr[2] == MAC_LO) )
				return (1);
		}
		else if ( mac==MAC_HI )
			return (1);

		return (0);
}

int dos_getc( void )					// use DOS to get key from console and not echo
{										// assumes character is available.
	regs.h.ah=DOS_GETKBD;
	int86( DOS_INT, &regs, &regs);
	return((int)regs.h.al);
}
/*
extern char far exe_share[];

void interrupt exe_int_handler(unsigned bp, unsigned di, unsigned si,
										  unsigned ds, unsigned es, unsigned dx,
										  unsigned cx, unsigned bx, unsigned ax,
										  unsigned ip, unsigned cs, unsigned flags)
{
//	printf("exe_share: %Fp", exe_share );
	if 	(ax==0) {
		bx=FP_OFF( exe_share );
		cx=FP_SEG( exe_share );
	}
	else {
		bx=FP_OFF( query );
		cx=FP_SEG( query );
	}

	return;
}
*/
