/*
 * WEBTCP.H
 *
 * Embedded Web Server
 * Copyright 1998,
 * JK microsystems, Inc. - ALL RIGHTS RESERVED
 * http://www.jkmicro.com
 *
 * THIS SOFTWARE IS NOT SHAREWARE, FREEWARE, OR PUBLIC DOMAIN.
 * IT IS THE PROPERTY OF JK microsystems.
 *
 * CUSTOMERS OF JK microsystems MAY MODIFY THE SOURCE CODE
 * AND/OR DISTRIBUTE THE BINARY IMAGE OF THIS SOFTWARE WITHOUT
 * ADDITIONAL COSTS PROVIDED IT IS RUN ONLY ON HARDWARE
 * MANUFACTURED BY JK microsystems.  ALL OTHER USE IS EXPRESSLY
 * PROHIBITED.
 *
 * THIS SOURCE CODE IS NOT TO BE DISCLOSED WITHOUT PRIOR APPROVAL
 * FROM JK microsystems.
 *
 * THIS PROGRAM IS DISTRIBUTED WITHOUT ANY WARRANTY;
 * WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY
 * OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * THE TCP/IP LIBRARIES ARE COPYRIGHT
 * UNIVERSITY OF WATERLOO.
 * THE LIBRARIES ARE PROVIDED FREE OF
 * CHARGE IN THEIR ORIGINAL DISTRIBUTION FORMAT.
 *
 */

/*
 * WATTCP library changes:
 *
 * The following changes are recommended.  They are
 * not reflected in the distributed library to
 * comply with the copyright statement.
 *
 * _arp_resolve() timeout in PCARP.C changed from 5 seconds
 *      to 3 seconds on line 292, comment read 'five seconds
 *      is long for ARP' changed to reduce dead time when peer
 *      cannot be resolved (ping another node).
 *
 */

/*
		1.0a    It all starts here - BETA release
				need to work on write speed, implement
				more HTTP functions, clean up header
				parsing, add watch dog, scripts.
				-EW 10 July 98

		1.0b    add GET method forms to embedded page
				HDWR to write / configure ports A and B
				add dynamic file support (int30, function 0x300)
				-EW 14 July 98

		1.0c    add embedded PING and PING.HTM pages.  use PING
				with GET method to request pings, view results
				on PING.HTM.  PING is not re-entrant.  a request
				will return 403 FORBIDDEN if the state machine
				is not idle.  the request will not disrupt
				the state machine or the queued requests.

				POST method forms should work w/ PING and HDWR.
				may not be the best...

				change PING and HDWR to be script files with
				a .SCR extension.

				fix bug in date routine (July reported as June)
				fix several allocation bugs (global constants)

				header receive/process could use a bit of work.
				seems solid, but sometimes doesnt get all of the
				header and gets an error.  probably needs a timeout
				or something...

				add -v (verbose) command line option.  normally has
				minimal console output.  w/ the -v option, logs
				all hits (and some other stuff) to console

				add mac # check /w int 15h - will only run
				on Flashlite w/ valid JK micro company ID

				-EW 31 July 98

		1.0d    fix bug in DF read, would only work on files
				smaller than READ_BLOCK.

				-EW 10 August 98

		1.0e	step back from 1.0d, no ping, works ok w/ DFS
				changes to write routines to check for available
				output buffer.  1.0d not released.

				!!!!! compile w/ makefile, not environment !!!!!

				-EW 3 September 98

		1.0f	rewrite header Rx code, was being stupid before.
				add big stack - messy but it *IS* there
				-EW 9 Sept 98

		1.1a	attempt at parsed HTML (int 31h), uses DFS file stuff.

		1.1b	after joels fixes
					problems:
						fprintf -> NULL
						using sizeof(buffer) when buffer is a char*
						no checking of sock_fastread
						fixed a inf loop dealing with _chk_sock...
				-EW 11 Nov 98
		2.0a
				user parsing
				-EW May 00
		2.0b
				add hook for user to look at get query string (USER.SCR)
				-EW May 01

*/

#ifndef __webtcp_h
#define __webtcp_h

#define VERSION "2.0b - User Function"

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <mem.h>
#include <string.h>
#include "con_io.h"
#include <dos.h>
#include <dir.h>
#include <time.h>
#include <bios.h>
#include <sys\stat.h>      		// for file statistics (fstat)
#include <ctype.h>

#include "tcp.h"
#include "status.h"

#define WEB_PORT 80				// common TCP/IP port for HTTP server
#define TZ_ENV "TZ=PST8PDT"		// default time zone environment
#define FALSE  0
#define TRUE !(FALSE)

#define READ_DISK 2				// bios read disk function
#define TEST_READY 16			// bios test disk ready
#define RESET 0					// bios disk reset

#define HEAD 1					// HTTP commands
#define GET 2
#define PUT 3
#define POST 4

#define PORT 1					// allow i/o port access from 'GET' type form, comment to disable

#ifdef PORT
#define PORT_A 0x60
#define PORT_B 0x61
#define PORT_AB_DIR 0x63
#define PORT_A_DIR_BIT 0x10
#define PORT_B_DIR_BIT 0x01
#endif


#define READ_BLOCK 1000				// block read from file, written to socket
#define RBUF (4096*sizeof(char))	// read buffer
#define TBUF (2048*sizeof(char))	// write buffer, must be >= socket tx buffer
#define WTBUF (40*sizeof(char))		// buffer for web_time

#define T_ESTAB 2					// timeout to wait for established connection after passive open
									// dont want to block everything
#define T_RXHEADER 3				// timeout for receiving HTTP header after connection established

#define SOCKETS 8					// number of sockets available.  only uses 1 at a time,
									// multiple sockets reduce problems due to delays when socket closes

#define DEF_PAGE "homepage.htm\0"	// default page
#define DEF_PATH "/WWW\0"	 		// default path

#define ABS(l) ((l>=0) ? l : -l)


#define EXE_INT 0x32       // interrupt to use for exe shared memory
#define EXE_SEG 0xCA			// location of exe table code segment in interrupt table (0x32 * 4 +2)
#define EXE_OFF 0xC8			// location of exe table offset in interrupt table (0x32 * 4)


void get_filetype ( char *ext, char *buffer, int *filetype);

#ifdef PORT

int proc_hdwr ( char *buffer );
// 		string must be of form port=(A,B,DIR_AB)&v=xx&v=xx&act=(READ,SET,CLEAR)
// 		xx: 0..9, A..F, hex value
// 		multiple V= strings are ANDed togethor to generate a mask for the operation

#endif

void fix_filename ( char *fn );			// change '\' to '/'

void print_all (char *buf, FILE *out ); // print string displaying /n and /r
void make_web_time (char * wt );        // format time/date per RFC-822
char *make_local_time (char * wt );        // create local time string, similar to RFC-822 format

void *proc_ping ( char *inbuf, char *outbuf, int *length, int *status, FILE *msg_out );
void ping_free ( char *ptr );			// free ping list when program dies

int flashlite( unsigned int mac );		// check if Flashlite ( JKmicro mac prefix )
int dos_getc( void );					// use dos to get keyboard w/o echo

//void df_tick ( void );				// call df to give it a time slice
char df_locate ( void );				// find DFS code segment and check for TSR signature and version
void huge *df ( char *filename, unsigned int *len, int *status );

void huge *parse_file ( FILE *fp, unsigned int *len, int *status );
void parse_free( void );

void interrupt exe_int_handler();

void user_init( int port );
int user_data( char far *out_str );
void user_close( void );
void user_store( void );
int user_sht ( char *buffer );

#endif  // end if def __webtcp_h
